ArenaHistory.ArenaMatch = {}

local ArenaMatch = ArenaHistory.ArenaMatch
local Player = ArenaHistory.Player
local Logger = ArenaHistory.Logger
local GameProto = ArenaHistory.GameProto

ArenaMatch.__index = ArenaMatch
ArenaMatch.logger = Logger.new("ArenaMatch")

ArenaMatch.MAX_PLAYERS = 5

local PB = LibStub("pblua").compile(GameProto.ast)

function ArenaMatch.new()
  local arena = setmetatable({}, ArenaMatch)
  arena.season = GetCurrentArenaSeason();
  arena.player = Player.new()
  arena.player.id = "player"
  arena.score = nil
  arena.has_seen_results = false
  arena.runTimeS = nil -- in seconds
  arena.startTimestamp = time()
  arena.myTeamMmr = nil
  arena.oppositeTeamMmr = nil
  arena.isRated = (select(2, IsActiveBattlefieldArena()) == 1) or
      (select(2, IsActiveBattlefieldArena()) == true)
  arena:updateZone()

  arena._playersByGUID = {}
  arena._playersById = {}
  for i = 1, ArenaMatch.MAX_PLAYERS do
    for _, prefix in pairs({"arena", "party"}) do
      local id = prefix .. i
      local player = Player.new()
      player.id = id
      arena._playersById[id] = player
    end
  end
  arena._playersById["player"] = arena.player
  arena:updatePlayers()

  arena.myTeam = {}
  arena.oppositeTeam = {}

  arena.addonVersion = nil
  arena.region = nil

  return arena
end

function ArenaMatch:updateZone()
  self.localizedMapName = GetRealZoneText()
end

function ArenaMatch:updatePlayer(player)
  if player == nil then
    return
  end
  ArenaMatch.logger:debug("updatePlayer %s", player.id)
  player:update()
  if player.guid ~= nil then
    self._playersByGUID[player.guid] = player
  end
end

function ArenaMatch:updatePlayers()
  ArenaMatch.logger:debug("updatePlayers")
  for _, player in pairs(self._playersById) do
    self:updatePlayer(player)
  end
end

function ArenaMatch:getPlayerByGUID(guid)
  if guid == nil then
    return nil
  end
  return self._playersByGUID[guid]
end

function ArenaMatch:getPlayerByUnitId(id)
  if id == nil then
    return nil
  end
  local player = self._playersById[id]
  if player ~= nil then
    return player
  end
  return self:getPlayerByGUID(UnitGUID(id))
end

function ArenaMatch:_readScoreBoard()
  ArenaMatch.logger:debug("Adding %d bg scores", GetNumBattlefieldScores())
  local greenTeam = {}
  local goldTeam = {}
  for i = 1, GetNumBattlefieldScores() do
    local player = Player.new()
    player:addBattlefieldScore(i)
    local name = player:getFullName()
    local teamIndex = select(6, GetBattlefieldScore(i))
    if teamIndex == 0 then
      greenTeam[name] = player
    else
      goldTeam[name] = player
    end
  end
  local myTeamIndex
  if greenTeam[self.player:getFullName()] ~= nil then
    self.myTeam = greenTeam
    self.oppositeTeam = goldTeam
    myTeamIndex = 0
  else
    self.myTeam = goldTeam
    self.oppositeTeam = greenTeam
    myTeamIndex = 1
  end
  local myTeamPlayers, oppositeTeamPlayers
  _, _, _, self.myTeamMmr, myTeamPlayers =
      GetBattlefieldTeamInfo(myTeamIndex)
  _, _, _, self.oppositeTeamMmr, oppositeTeamPlayers =
      GetBattlefieldTeamInfo(1 - myTeamIndex)
  self.teamSize = math.max(myTeamPlayers, oppositeTeamPlayers)
  self.has_seen_results = true  
  local winner = GetBattlefieldWinner()
  if winner == nil then
    self.score = 0
  else
    if winner == myTeamIndex then
      self.score = 1
    elseif winner == 1 - myTeamIndex then
      self.score = -1
    else
      self.score = 0
    end
  end
  self.runTimeS = ceil(GetBattlefieldInstanceRunTime() / 1000)
end

function ArenaMatch:finish()
  ArenaMatch.logger:debug("finish")
  self:updatePlayers()

  for _, player in pairs(self._playersById) do
    player:clearCC()
  end

  if GetNumBattlefieldScores() ~= 0 then
    self:_readScoreBoard()     
    for _, player in pairs(self._playersById) do
      if player.name ~= nil and player.server ~= nil and
          player.specId ~= nil then
        local name = player:getFullName()
        if player:getArenaId() ~= nil and self.oppositeTeam[name] ~= nil then
          self.oppositeTeam[name]:merge(player)
        end
        if player:getArenaId() == nil and self.myTeam[name] ~= nil then
          self.myTeam[name]:merge(player)
        end
      end
    end
  else
    ArenaMatch.logger:warning("Can't read scoreboard")
    for _, player in pairs(self._playersById) do
      if player.specId ~= nil then
        if player:getArenaId() ~= nil then
          table.insert(self.oppositeTeam, player)
        else
          table.insert(self.myTeam, player)
        end
      end
    end
  end

  if self.score == nil then
    self.score = -1
  end
  if self.runTimeS == nil then
    self.runTimeS = ceil(time() - self.startTimestamp)
  end
  if self.teamSize == nil then
    self.teamSize = math.max(table.getn(self.myTeam),
        table.getn(self.oppositeTeam))
  end
end

function ArenaMatch:getFirstDied()
  local firstDiedPlayer = nil
  for _, player in pairs(self._playersById) do
    if player.died_timestamp ~= nil then
      if firstDiedPlayer == nil or
          player.died_timestamp < firstDiedPlayer.died_timestamp then
        firstDiedPlayer = player
      end
    end
  end
  return firstDiedPlayer ~= nil and firstDiedPlayer.id or nil
end

function ArenaMatch:dump(isProto)
  ArenaMatch.logger:debug("Ready to dump %d vs %d Arena match",
      self.teamSize, self.teamSize)
  local dump
  if isProto then
    dump = PB.Game()
  else
    dump = {}
  end
  dump.season = self.season
  dump.score = self.score
  dump.has_seen_results = self.has_seen_results
  dump.run_time_s = self.runTimeS
  dump.is_rated = self.isRated
  dump.team_size = self.teamSize
  dump.localized_map_name = self.localizedMapName
  dump.start_timestamp = self.startTimestamp
  dump.my_team_mmr = self.myTeamMmr
  dump.opposite_team_mmr = self.oppositeTeamMmr
  dump.region = self.region
  dump.addon_version = self.addonVersion
  dump.first_died_id = self:getFirstDied()
  dump.my_team = {}
  local myTeamSize = 0
  for _, player in pairs(self.myTeam) do
    table.insert(dump.my_team, player:dump(isProto))
    myTeamSize = myTeamSize + 1
  end
  dump.opposite_team = {}
  local oppositeTeamSize = 0
  for _, player in pairs(self.oppositeTeam) do
    table.insert(dump.opposite_team, player:dump(isProto))
    oppositeTeamSize = oppositeTeamSize + 1
  end

  if myTeamSize > 0 and (myTeamSize ~= self.teamSize or
      oppositeTeamSize ~= self.teamSize) then
    ArenaMatch.logger:warning("Not enough players, my: %d, enemy: %d, " ..
        "required: %d", myTeamSize, oppositeTeamSize, self.teamSize)
    return nil
  end

  return dump     
end
